@@ -3,12 +3,14 @@
 """
 Tests for `attr._make`.
 """
+
 import copy
 import functools
 import gc
 import inspect
 import itertools
 import sys
+import unicodedata
 
 from operator import attrgetter
 from typing import Generic, TypeVar
@@ -19,11 +21,13 @@ from hypothesis import assume, given
 from hypothesis.strategies import booleans, integers, lists, sampled_from, text
 
 import attr
+import attrs
 
 from attr import _config
-from attr._compat import PY_3_8_PLUS, PY_3_10_PLUS, PY_3_14_PLUS
+from attr._compat import PY_3_10_PLUS
 from attr._make import (
     Attribute,
+    ClassProps,
     Factory,
     _AndValidator,
     _Attributes,
@@ -53,7 +57,9 @@ from .strategies import (
 from .utils import simple_attr
 
 
-attrs_st = simple_attrs.map(lambda c: Attribute.from_counting_attr("name", c))
+attrs_st = simple_attrs.map(
+    lambda c: Attribute.from_counting_attr("name", c, False)
+)
 
 
 @pytest.fixture(name="with_and_without_validation", params=[True, False])
@@ -177,7 +183,7 @@ class TestTransformAttrs:
         Does not attach __attrs_attrs__ to the class.
         """
         C = make_tc()
-        _transform_attrs(C, None, False, False, True, None)
+        _transform_attrs(C, None, False, ClassProps.KeywordOnly.NO, True, None)
 
         assert None is getattr(C, "__attrs_attrs__", None)
 
@@ -186,7 +192,9 @@ class TestTransformAttrs:
         Transforms every `_CountingAttr` and leaves others (a) be.
         """
         C = make_tc()
-        attrs, _, _ = _transform_attrs(C, None, False, False, True, None)
+        attrs, _, _ = _transform_attrs(
+            C, None, False, ClassProps.KeywordOnly.NO, True, None
+        )
 
         assert ["z", "y", "x"] == [a.name for a in attrs]
 
@@ -199,8 +207,8 @@ class TestTransformAttrs:
         class C:
             pass
 
-        assert _Attributes(((), [], {})) == _transform_attrs(
-            C, None, False, False, True, None
+        assert _Attributes((), [], {}) == _transform_attrs(
+            C, None, False, ClassProps.KeywordOnly.NO, True, None
         )
 
     def test_transforms_to_attribute(self):
@@ -209,7 +217,7 @@ class TestTransformAttrs:
         """
         C = make_tc()
         attrs, base_attrs, _ = _transform_attrs(
-            C, None, False, False, True, None
+            C, None, False, ClassProps.KeywordOnly.NO, True, None
         )
 
         assert [] == base_attrs
@@ -227,7 +235,9 @@ class TestTransformAttrs:
             y = attr.ib()
 
         with pytest.raises(ValueError) as e:
-            _transform_attrs(C, None, False, False, True, None)
+            _transform_attrs(
+                C, None, False, ClassProps.KeywordOnly.NO, True, None
+            )
         assert (
             "No mandatory attributes allowed after an attribute with a "
             "default value or factory.  Attribute in question: Attribute"
@@ -240,11 +250,8 @@ class TestTransformAttrs:
 
     def test_kw_only(self):
         """
-        Converts all attributes, including base class' attributes, if `kw_only`
-        is provided. Therefore, `kw_only` allows attributes with defaults to
-        precede mandatory attributes.
-
-        Updates in the subclass *don't* affect the base class attributes.
+        Converts all attributes in the current class if `kw_only` is provided.
+        Base class attributes are **not** affected.
         """
 
         @attr.s
@@ -259,7 +266,25 @@ class TestTransformAttrs:
             y = attr.ib()
 
         attrs, base_attrs, _ = _transform_attrs(
-            C, None, False, True, True, None
+            C, None, False, ClassProps.KeywordOnly.YES, True, None
+        )
+
+        assert len(attrs) == 3
+        assert len(base_attrs) == 1
+
+        for a in attrs:
+            if a.name == "b":
+                assert a.kw_only is False
+            else:
+                assert a.kw_only is True
+
+        attrs, base_attrs, _ = _transform_attrs(
+            C,
+            None,
+            False,
+            ClassProps.KeywordOnly.FORCE,
+            True,
+            None,
         )
 
         assert len(attrs) == 3
@@ -283,7 +308,7 @@ class TestTransformAttrs:
             y = attr.ib()
 
         attrs, base_attrs, _ = _transform_attrs(
-            C, {"x": attr.ib()}, False, False, True, None
+            C, {"x": attr.ib()}, False, ClassProps.KeywordOnly.NO, True, None
         )
 
         assert [] == base_attrs
@@ -484,6 +509,77 @@ class TestAttributes:
         assert "x" == C.__attrs_attrs__[0].name
         assert all(isinstance(a, Attribute) for a in C.__attrs_attrs__)
 
+    def test_sets_attrs_props(self):
+        """
+        Sets the `__attrs_props__` class attribute with the effective decorator
+        properties.
+        """
+
+        @attr.s(
+            slots=True,
+            frozen=True,
+            repr=True,
+            eq=True,
+            order=True,
+            unsafe_hash=True,
+            init=True,
+            match_args=False,
+            kw_only=True,
+            auto_attribs=True,
+            cache_hash=True,
+            str=True,
+        )
+        class C:
+            x: int = attr.ib()
+
+        assert ClassProps(
+            is_exception=False,
+            is_slotted=True,
+            is_frozen=True,
+            added_init=True,
+            added_repr=True,
+            added_eq=True,
+            added_ordering=True,
+            hashability=ClassProps.Hashability.HASHABLE_CACHED,
+            added_match_args=False,
+            kw_only=ClassProps.KeywordOnly.FORCE,
+            has_weakref_slot=True,
+            collected_fields_by_mro=False,
+            added_str=True,
+            added_pickling=True,
+            on_setattr_hook=None,
+            field_transformer=None,
+        ) == attrs.inspect(C)
+
+    def test_sets_attrs_props_defaults(self):
+        """
+        Default values are derived in `__attrs_props__` when not specified in
+        the decorator.
+        """
+
+        @attr.s
+        class CDef:
+            x = attr.ib()
+
+        assert ClassProps(
+            is_exception=False,
+            is_slotted=False,
+            is_frozen=False,
+            added_init=True,
+            added_repr=True,
+            added_eq=True,
+            added_ordering=True,
+            hashability=ClassProps.Hashability.UNHASHABLE,
+            added_match_args=True,
+            kw_only=ClassProps.KeywordOnly.NO,
+            has_weakref_slot=True,
+            collected_fields_by_mro=False,
+            added_str=False,
+            added_pickling=False,
+            on_setattr_hook=None,
+            field_transformer=None,
+        ) == attrs.inspect(CDef)
+
     def test_empty(self):
         """
         No attributes, no problems.
@@ -713,6 +809,49 @@ class TestAttributes:
 
         assert 3 == val == inst.kw_and_default
 
+    @pytest.mark.usefixtures("with_and_without_validation")
+    def test_pre_init_with_mixture_of_defaults_and_kw_only(self):
+        """
+        Attrs should properly handle a mixture of positional, positional with
+        default, keyword-only, and keyword-only with default attributes when
+        passing values to __attrs_pre_init__.
+        """
+        g_val1 = None
+        g_val2 = None
+        g_val3 = None
+        g_val4 = None
+        g_val5 = None
+        g_val6 = None
+
+        @attr.define
+        class MixtureClass:
+            val1: int
+            val2: int = 100
+            val3: int = attr.field(factory=int)
+            val4: int = attr.field(kw_only=True)
+            val5: int = attr.field(default=100, kw_only=True)
+            val6: int = attr.field(factory=int, kw_only=True)
+
+            def __attrs_pre_init__(self, val1, val2, val3, val4, val5, val6):
+                nonlocal g_val1, g_val2, g_val3, g_val4, g_val5, g_val6
+                g_val1 = val1
+                g_val2 = val2
+                g_val3 = val3
+                g_val4 = val4
+                g_val5 = val5
+                g_val6 = val6
+
+        inst = MixtureClass(
+            val1=200, val2=200, val3=200, val4=200, val5=200, val6=200
+        )
+
+        assert 200 == g_val1 == inst.val1
+        assert 200 == g_val2 == inst.val2
+        assert 200 == g_val3 == inst.val3
+        assert 200 == g_val4 == inst.val4
+        assert 200 == g_val5 == inst.val5
+        assert 200 == g_val6 == inst.val6
+
     @pytest.mark.usefixtures("with_and_without_validation")
     def test_post_init(self):
         """
@@ -973,8 +1112,8 @@ class TestKeywordOnlyAttributes:
 
     def test_keyword_only_class_level_subclassing(self):
         """
-        Subclass `kw_only` propagates to attrs inherited from the base,
-        allowing non-default following default.
+        When `force_kw_only=True`, subclass `kw_only` propagates to attrs
+        inherited from the base, allowing non-default following default.
         """
 
         @attr.s
@@ -987,12 +1126,23 @@ class TestKeywordOnlyAttributes:
 
         with pytest.raises(TypeError):
             C(1)
+        with pytest.raises(TypeError):
+            C(0, y=1)
 
         c = C(x=0, y=1)
 
         assert c.x == 0
         assert c.y == 1
 
+        @attr.s(kw_only=True, force_kw_only=False)
+        class C(Base):
+            y = attr.ib()
+
+        c = C(0, y=1)
+
+        assert c.x == 0
+        assert c.y == 1
+
     def test_init_false_attribute_after_keyword_attribute(self):
         """
         A positional attribute cannot follow a `kw_only` attribute,
@@ -1049,6 +1199,144 @@ class TestKeywordOnlyAttributes:
         assert c.non_init_function_default == "ab"
         assert c.non_init_keyword_default == "default-by-keyword"
 
+    def test_attr_level_kw_only_and_class_level_kw_only(self):
+        """
+        If a field explicitly sets 'kw_only=False', it should not be made
+        keyword-only even if the class sets 'kw_only=True'.
+        """
+
+        @attr.s(kw_only=True)
+        class OldClassOldBehavior:
+            yes = attr.ib()
+            no = attr.ib(kw_only=False)
+
+        @attr.s(kw_only=True, force_kw_only=False)
+        class OldClassNewBehavior:
+            yes = attr.ib()
+            no = attr.ib(kw_only=False)
+
+        @attr.define(kw_only=True, force_kw_only=True)
+        class NewClassOldBehavior:
+            yes = attr.field()
+            no = attr.field(kw_only=False)
+
+        @attr.define(kw_only=True)
+        class NewClassNewBehavior:
+            yes = attr.field()
+            no = attr.field(kw_only=False)
+
+        for cls in [OldClassNewBehavior, NewClassNewBehavior]:
+            fs = fields_dict(cls)
+            assert fs["yes"].kw_only is True
+            assert fs["no"].kw_only is False
+
+        for cls in [OldClassOldBehavior, NewClassOldBehavior]:
+            fs = fields_dict(cls)
+
+            assert fs["yes"].kw_only is True
+            assert fs["no"].kw_only is True
+
+    def test_kw_only_inheritance(self):
+        """
+        Comprehensive test about how `kw_only` works when there's multiple
+        levels of inheritance with different `kw_only` settings.
+        """
+
+        @attr.define()
+        class A:
+            a = attr.field()
+            a_t = attr.field(kw_only=True)
+            a_f = attr.field(kw_only=False)
+
+        @attr.define(kw_only=True)
+        class B(A):
+            b = attr.field()
+            b_t = attr.field(kw_only=True)
+            b_f = attr.field(kw_only=False)
+
+        @attr.define(kw_only=False)
+        class C(B):
+            c = attr.field()
+            c_t = attr.field(kw_only=True)
+            c_f = attr.field(kw_only=False)
+
+        fs = fields_dict(A)
+
+        assert fs["a"].kw_only is False
+        assert fs["a_t"].kw_only is True
+        assert fs["a_f"].kw_only is False
+
+        fs = fields_dict(B)
+
+        assert fs["a"].kw_only is False
+        assert fs["a_t"].kw_only is True
+        assert fs["a_f"].kw_only is False
+        assert fs["b"].kw_only is True
+        assert fs["b_t"].kw_only is True
+        assert fs["b_f"].kw_only is False
+
+        fs = fields_dict(C)
+
+        assert fs["a"].kw_only is False
+        assert fs["a_t"].kw_only is True
+        assert fs["a_f"].kw_only is False
+        assert fs["b"].kw_only is True
+        assert fs["b_t"].kw_only is True
+        assert fs["b_f"].kw_only is False
+        assert fs["c"].kw_only is False
+        assert fs["c_t"].kw_only is True
+        assert fs["c_f"].kw_only is False
+
+    def test_kw_only_inheritance_force_kw_only(self):
+        """
+        Similar to above, but when `force_kw_only` (pre-25.3.0 behavior).
+        """
+
+        @attr.define(force_kw_only=True)
+        class A:
+            a = attr.field()
+            a_t = attr.field(kw_only=True)
+            a_f = attr.field(kw_only=False)
+
+        @attr.define(kw_only=True, force_kw_only=True)
+        class B(A):
+            b = attr.field()
+            b_t = attr.field(kw_only=True)
+            b_f = attr.field(kw_only=False)
+
+        @attr.define(kw_only=False, force_kw_only=True)
+        class C(B):
+            c = attr.field()
+            c_t = attr.field(kw_only=True)
+            c_f = attr.field(kw_only=False)
+
+        fs = fields_dict(A)
+
+        assert fs["a"].kw_only is False
+        assert fs["a_t"].kw_only is True
+        assert fs["a_f"].kw_only is False
+
+        fs = fields_dict(B)
+
+        assert fs["a"].kw_only is True
+        assert fs["a_t"].kw_only is True
+        assert fs["a_f"].kw_only is True
+        assert fs["b"].kw_only is True
+        assert fs["b_t"].kw_only is True
+        assert fs["b_f"].kw_only is True
+
+        fs = fields_dict(C)
+
+        assert fs["a"].kw_only is False
+        assert fs["a_t"].kw_only is True
+        assert fs["a_f"].kw_only is False
+        assert fs["b"].kw_only is True
+        assert fs["b_t"].kw_only is True
+        assert fs["b_f"].kw_only is True
+        assert fs["c"].kw_only is False
+        assert fs["c_t"].kw_only is True
+        assert fs["c_f"].kw_only is False
+
 
 @attr.s
 class GC:
@@ -1099,6 +1387,48 @@ class TestMakeClass:
 
         assert repr(C(1)).startswith("<tests.test_make.C object at 0x")
 
+    def test_normalized_unicode_attr_args(self):
+        """
+        Unicode identifiers are valid in Python.
+        """
+        clsname = "ü"
+
+        assert clsname == unicodedata.normalize("NFKC", clsname)
+
+        attrname = "ß"
+
+        assert attrname == unicodedata.normalize("NFKC", attrname)
+
+        C = make_class(clsname, [attrname], repr=False)
+
+        assert repr(C(1)).startswith("<tests.test_make.ü object at 0x")
+
+        kwargs = {"ß": 1}
+        c = C(**kwargs)
+
+        assert 1 == c.ß
+
+    def test_unnormalized_unicode_attr_args(self):
+        """
+        Unicode identifiers are normalized to NFKC form in Python.
+        """
+
+        clsname = "Ŀ"
+
+        assert clsname != unicodedata.normalize("NFKC", clsname)
+
+        attrname = "ㅁ"
+
+        assert attrname != unicodedata.normalize("NFKC", attrname)
+
+        C = make_class(clsname, [attrname], repr=False)
+        assert repr(C(1)).startswith("<tests.test_make.L· object at 0x")
+
+        kwargs = {unicodedata.normalize("NFKC", attrname): 1}
+        c = C(**kwargs)
+
+        assert 1 == c.ㅁ
+
     def test_catches_wrong_attrs_type(self):
         """
         Raise `TypeError` if an invalid type for attrs is passed.
@@ -1673,17 +2003,25 @@ class TestClassBuilder:
             C,
             None,
             True,
-            True,
-            False,
-            False,
-            False,
-            False,
-            False,
-            False,
-            True,
-            None,
-            False,
-            None,
+            ClassProps(
+                is_exception=False,
+                is_slotted=True,
+                is_frozen=True,
+                added_init=True,
+                added_repr=True,
+                added_eq=True,
+                added_ordering=False,
+                hashability=ClassProps.Hashability.UNHASHABLE,
+                added_match_args=True,
+                kw_only=ClassProps.KeywordOnly.NO,
+                has_weakref_slot=False,
+                collected_fields_by_mro=True,
+                added_str=False,
+                added_pickling=True,
+                on_setattr_hook=None,
+                field_transformer=None,
+            ),
+            has_custom_setattr=False,
         )
 
         assert "<_ClassBuilder(cls=C)>" == repr(b)
@@ -1699,18 +2037,26 @@ class TestClassBuilder:
         b = _ClassBuilder(
             C,
             None,
-            True,
-            True,
             False,
-            False,
-            False,
-            False,
-            False,
-            False,
-            True,
-            None,
-            False,
-            None,
+            ClassProps(
+                is_exception=False,
+                is_slotted=True,
+                is_frozen=True,
+                added_init=True,
+                added_repr=True,
+                added_eq=True,
+                added_ordering=False,
+                hashability=ClassProps.Hashability.UNHASHABLE,
+                added_match_args=True,
+                kw_only=ClassProps.KeywordOnly.NO,
+                has_weakref_slot=False,
+                collected_fields_by_mro=True,
+                added_str=False,
+                added_pickling=True,
+                on_setattr_hook=None,
+                field_transformer=None,
+            ),
+            has_custom_setattr=False,
         )
 
         cls = (
@@ -1734,7 +2080,6 @@ class TestClassBuilder:
             "__repr__",
             "__str__",
             "__eq__",
-            "__ne__",
             "__lt__",
             "__le__",
             "__gt__",
@@ -1766,7 +2111,9 @@ class TestClassBuilder:
         organic_prefix = C.organic.__qualname__.rsplit(".", 1)[0]
         assert organic_prefix + "." + meth_name == meth_C.__qualname__
 
-    def test_handles_missing_meta_on_class(self):
+    def test_handles_missing_meta_on_class(
+        self, monkeypatch: pytest.MonkeyPatch
+    ):
         """
         If the class hasn't a __module__ or __qualname__, the method hasn't
         either.
@@ -1775,23 +2122,43 @@ class TestClassBuilder:
         class C:
             pass
 
+        orig_hasattr = __builtins__["hasattr"]
+
+        def our_hasattr(obj, name, /) -> bool:
+            if name in ("__module__", "__qualname__"):
+                return False
+            return orig_hasattr(obj, name)
+
+        monkeypatch.setitem(
+            _ClassBuilder.__init__.__globals__["__builtins__"],
+            "hasattr",
+            our_hasattr,
+        )
+
         b = _ClassBuilder(
             C,
             these=None,
-            slots=False,
-            frozen=False,
-            weakref_slot=True,
-            getstate_setstate=False,
             auto_attribs=False,
-            is_exc=False,
-            kw_only=False,
-            cache_hash=False,
-            collect_by_mro=True,
-            on_setattr=None,
+            props=ClassProps(
+                is_exception=False,
+                is_slotted=False,
+                is_frozen=False,
+                added_init=True,
+                added_repr=True,
+                added_eq=True,
+                added_ordering=False,
+                hashability=ClassProps.Hashability.UNHASHABLE,
+                added_match_args=True,
+                kw_only=ClassProps.KeywordOnly.NO,
+                has_weakref_slot=True,
+                collected_fields_by_mro=True,
+                added_str=False,
+                added_pickling=True,
+                on_setattr_hook=None,
+                field_transformer=None,
+            ),
             has_custom_setattr=False,
-            field_transformer=None,
         )
-        b._cls = {}  # no __module__; no __qualname__
 
         def fake_meth(self):
             pass
@@ -1799,6 +2166,8 @@ class TestClassBuilder:
         fake_meth.__module__ = "42"
         fake_meth.__qualname__ = "23"
 
+        b._cls = {}  # No module and qualname
+
         rv = b._add_method_dunders(fake_meth)
 
         assert "42" == rv.__module__ == fake_meth.__module__
@@ -1837,8 +2206,6 @@ class TestClassBuilder:
 
         assert [C2] == C.__subclasses__()
 
-    @pytest.mark.skipif(not PY_3_8_PLUS, reason="cached_property is 3.8+")
-    @pytest.mark.xfail(PY_3_14_PLUS, reason="Currently broken on nightly.")
     def test_no_references_to_original_when_using_cached_property(self):
         """
         When subclassing a slotted class and using cached property, there are
@@ -2091,7 +2458,7 @@ class TestDetermineAttrsEqOrder:
         eq=False, order=True raises a meaningful ValueError.
         """
         with pytest.raises(
-            ValueError, match="`order` can only be True if `eq` is True too."
+            ValueError, match="`order` can only be True if `eq` is True too"
         ):
             _determine_attrs_eq_order(None, False, True, True)
 
@@ -2103,7 +2470,7 @@ class TestDetermineAttrsEqOrder:
         assume(eq is not None or order is not None)
 
         with pytest.raises(
-            ValueError, match="Don't mix `cmp` with `eq' and `order`."
+            ValueError, match="Don't mix `cmp` with `eq' and `order`"
         ):
             _determine_attrs_eq_order(cmp, eq, order, True)
 
@@ -2158,7 +2525,7 @@ class TestDetermineAttribEqOrder:
         eq=False, order=True raises a meaningful ValueError.
         """
         with pytest.raises(
-            ValueError, match="`order` can only be True if `eq` is True too."
+            ValueError, match="`order` can only be True if `eq` is True too"
         ):
             _determine_attrib_eq_order(None, False, True, True)
 
@@ -2170,7 +2537,7 @@ class TestDetermineAttribEqOrder:
         assume(eq is not None or order is not None)
 
         with pytest.raises(
-            ValueError, match="Don't mix `cmp` with `eq' and `order`."
+            ValueError, match="Don't mix `cmp` with `eq' and `order`"
         ):
             _determine_attrib_eq_order(cmp, eq, order, True)
 
@@ -2182,7 +2549,6 @@ class TestDocs:
             "__init__",
             "__repr__",
             "__eq__",
-            "__ne__",
             "__lt__",
             "__le__",
             "__gt__",
